from datetime import date, datetime
from typing import Dict, Any, List, Set, Union, Optional
from requests import Response
from requests.auth import HTTPBasicAuth
import sys
from .broker import make_request
from .helpers import check_required_env_vars, combine_env_configs, validate_date_string


env_config: Dict[str, Any] = combine_env_configs()


def twilio_lookup(phone_number: str, data_packages: list = [], **kwargs: Dict[str, Any]) -> Response:
    """query information on a phone number so that you can make a trusted interaction with your user.
        With this endpoint, you can format and validate phone numbers with the free Basic Lookup request
        and add on data packages to get even more in-depth carrier and caller information.

    Args:
        phone_number (str): The phone number to look up
        data_packages (list): A Python list of fields to return. Possible values are validation,
            caller_name, sim_swap, call_forwarding, line_status, line_type_intelligence, identity_match,
            reassigned_number, sms_pumping_risk, phone_number_quality_score, pre_fill.

    Returns:
        Response: requests.Response json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'TWILIO_API_SID',
        'TWILIO_API_SECRET'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    # Valid set of data packages for Twilio. Compare the ones that the user passed in
    # to ensure that they've passed valid ones. Exit immediately if they didn't.
    valid_data_packages: Set = {'caller_name', 'sim_swap', 'call_forwarding', 'line_status',
                                'line_type_intelligence', 'identity_match', 'reassigned_number',
                                'sms_pumping_risk', 'phone_number_quality_score', 'pre_fill'}
    data_packages_set: Set = set(data_packages)
    invalid_packages = data_packages_set - valid_data_packages
    if len(invalid_packages) != 0:
        print(f'[!] Error: "{", ".join(invalid_packages)}" are not valid data packages. Valid '
              f'packages include {", ".join(valid_data_packages)}', file=sys.stderr)
        sys.exit(1)

    method: str = 'get'
    url: str = f'https://lookups.twilio.com/v2/PhoneNumbers/{phone_number}'

    auth = HTTPBasicAuth(env_config['TWILIO_API_SID'], env_config['TWILIO_API_SECRET'])

    params: Dict = {
        'Fields': ','.join(data_packages),
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        auth=auth,
        params=params,
        timeout=kwargs.get("timeout")
    )

    return result


def twilio_usage_report(start_date: Union[str, date],
                        end_date: Optional[Union[str, date]] = None) -> Response:
    """Return a usage report for all activities between the start_date and end_date.

    Args:
        start_date (Union[str, date]): Only include usage that has occurred on or after this
            date. Specify the date in GMT and format as YYYY-MM-DD
        end_date (Optional[Union[str, date]], optional): Only include usage that occurred on
            or before this date. Specify the date in GMT and format as YYYY-MM-DD. Defaults to None.

    Returns:
        Response: requests.Response json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'TWILIO_ACCOUNT_SID',
        'TWILIO_API_SID',
        'TWILIO_API_SECRET'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    if end_date is None:
        end_date: str = datetime.now().strftime("%Y-%m-%d")

    if not validate_date_string(start_date) or not validate_date_string(end_date):
        print(f'[!] Error: One of your start date {start_date} or end date {end_date} '
              'does not match the format YYYY-MM-DD')
        sys.exit()

    method: str = 'get'
    url: str = f'https://api.twilio.com/2010-04-01/Accounts/{env_config["TWILIO_ACCOUNT_SID"]}/Usage/Records.json'

    auth = HTTPBasicAuth(env_config['TWILIO_API_SID'], env_config['TWILIO_API_SECRET'])

    params: Dict = {
        'StartDate': start_date,
        'EndDate': end_date
    }

    result: Response = make_request(
        method=method,
        url=url,
        auth=auth,
        params=params
    )

    return result



from typing import Dict, Any, List
from requests import Response
from .broker import make_request
from .helpers import check_required_env_vars, combine_env_configs


env_config: Dict[str, Any] = combine_env_configs()


def flashpoint_search_communities(query: str, **kwargs: Dict[str, Any]) -> Response:
    """Communities Search allows search requests over article and conversation data.
    Article data is made up of things like blogs and paste sites. Conversation data
    is made up of chats, forums, and social media posts.

    Args:
        query (str): A word or phrase to search.

    Returns:
        Response: requests.Response object from the request
    """

    required_vars: List[str] = [
        'FLASHPOINT_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'post'
    url: str = 'https://api.flashpoint.io/sources/v2/communities'
    headers: Dict = {
        'accept': 'application/json',
        'content-type': 'application/json',
        'Authorization': f'Bearer {env_config["FLASHPOINT_API_KEY"]}'
    }
    payload: Dict = {
        'query': query,
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        json=payload,
        timeout=kwargs.get("timeout")
    )

    return result


def flashpoint_search_fraud(query: str, **kwargs: Dict[str, Any]) -> Response:
    """Fraud search helps fraud teams at card-issuing banks detect compromised
    credit cards from illicit communities and data breaches. It also helps identify
    high-risk merchants over our different card fraud data types.

    Args:
        query (str): A word or phrase to search

    Returns:
        Response: requests.Response object from the request
    """

    required_vars: List[str] = [
        'FLASHPOINT_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'post'
    url: str = 'https://api.flashpoint.io/sources/v2/fraud'
    headers: Dict = {
        'accept': 'application/json',
        'content-type': 'application/json',
        'Authorization': f'Bearer {env_config["FLASHPOINT_API_KEY"]}'
    }
    payload: Dict = {
        'query': query,
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        json=payload,
        timeout=kwargs.get("timeout")
    )

    return result


def flashpoint_search_marketplaces(query: str, **kwargs: Dict[str, Any]) -> Response:

    """Markets search allows search requests over our different market places data.

    Args:
        query (str): A word or phrase to search

    Returns:
        Response: requests.Response object from the request
    """

    required_vars: List[str] = [
        'FLASHPOINT_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'post'
    url: str = 'https://api.flashpoint.io/sources/v2/markets'
    headers: Dict = {
        'accept': 'application/json',
        'content-type': 'application/json',
        'Authorization': f'Bearer {env_config["FLASHPOINT_API_KEY"]}'
    }
    payload: Dict = {
        'query': query,
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        json=payload,
        timeout=kwargs.get("timeout")
    )

    return result


def flashpoint_search_media(query: str, **kwargs: Dict[str, Any]) -> Response:
    """Media search allows search requests over our media data, specifically
    media that have been through our Optical Character Recogintion (OCR) process.
    Once media have been through our OCR process, any text, classifications, or
    logos found within the media are available for search.

    Args:
        query (str): A word or phrase to search.

    Returns:
        Response: requests.Response object from the request
    """

    required_vars: List[str] = [
        'FLASHPOINT_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'post'
    url: str = 'https://api.flashpoint.io/sources/v2/media'
    headers: Dict = {
        'accept': 'application/json',
        'content-type': 'application/json',
        'Authorization': f'Bearer {env_config["FLASHPOINT_API_KEY"]}'
    }
    payload: Dict = {
        'query': query,
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        json=payload,
        timeout=kwargs.get("timeout")
    )

    return result


def flashpoint_get_media_object(id: str) -> Response:
    """Media ID request allows users to directly lookup the document based on the media ID provided.

    Args:
        id (str): the id of the media object to retrieve

    Returns:
        Response: requests.Response object from the request
    """

    required_vars: List[str] = [
        'FLASHPOINT_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://api.flashpoint.io/sources/v2/media/{id}'
    headers: Dict = {
        'accept': 'application/json',
        'content-type': 'application/json',
        'Authorization': f'Bearer {env_config["FLASHPOINT_API_KEY"]}'
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers
    )

    return result


def flashpoint_get_media_image(storage_uri: str) -> Response:
    """Download the media from a media object by its storage_uri field

    Args:
        storage_uri (str): the storage_uri field from the media object

    Returns:
        Response: requests.Response object from the request
    """

    required_vars: List[str] = [
        'FLASHPOINT_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = 'https://api.flashpoint.io/sources/v1/media/'
    headers: Dict = {
        'accept': 'application/json',
        'content-type': 'application/json',
        'Authorization': f'Bearer {env_config["FLASHPOINT_API_KEY"]}'
    }

    params = {
        "asset_id": storage_uri
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        params=params
    )

    return result



from typing import Dict, Any, List
from requests import Response
import sys
from .broker import make_request
from .helpers import check_required_env_vars, combine_env_configs


env_config: Dict[str, Any] = combine_env_configs()


def spycloud_sip_cookie_domains(cookie_domains: str, **kwargs: Dict[str, Any]) -> Response:
    """Return botnet sourced cookie data for your domain and its subdomains

    Args:
        cookie_domains (str): This parameter allows you to define a cookie \
            domain to search against, results will include all subdomains. \
            Optionally, a specific cookie subdomain could be used which will \
            result in only that specific cookie subdomain returned.

    Returns:
        Response: requests.Response object from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'SPYCLOUD_API_SIP_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://api.spycloud.io/sip-v1/breach/data/cookie-domains/{cookie_domains}'
    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_SIP_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(method=method, url=url, headers=headers, params=params)

    return result


def spycloud_ato_breach_catalog(query: str, **kwargs: Dict[str, Any]) -> Response:
    """List or Query the Breach Catalog

    Args:
        query (str): Query value to search the breach catalog for.

    Returns:
        Response: requests.Response object from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'SPYCLOUD_API_ATO_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = 'https://api.spycloud.io/sp-v2/breach/catalog'
    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_ATO_KEY']
    }
    params: Dict = {
        'query': query,
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        params=params,
        timeout=kwargs.get("timeout")
    )

    return result


def spycloud_ato_search(search_type: str, query: str, **kwargs: Dict[str, Any]) -> Response:
    """Perform search against Spycloud's Consumer ATO API to query its vast collection of
        breach records and surrounding metadata

    Args:
        search_type (str): can be one of domain, email, ip, username, or phone-number
        query (str): the search query

    Returns:
        Response: requests.Response object from the request
    """

    required_vars: List[str] = [
        'SPYCLOUD_API_ATO_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    # These are valid endpoints and their corresponding full URLs. We'll use these
    # to check that the user passed a valid 'search_type' parameter
    base_url: str = 'https://api.spycloud.io/sp-v2/breach/data'
    valid_endpoints: Dict[str, str] = {
        'domain': f'{base_url}/domains',
        'email': f'{base_url}/emails',
        'ip': f'{base_url}/ips',
        'username': f'{base_url}/usernames',
        'phone-number': f'{base_url}/phone-numbers',
    }

    # Completely exit if they supply an invalid search_type
    if search_type not in valid_endpoints:
        print(f'[!] Error: "{search_type}" is not a valid search type. Must be one of '
              f'{", ".join(valid_endpoints.keys())}', file=sys.stderr)
        sys.exit(1)

    method: str = 'get'
    url: str = f'{valid_endpoints[search_type]}/{query}'

    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_ATO_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        params=params,
        timeout=kwargs.get("timeout")
    )

    return result


def spycloud_inv_search(search_type: str, query: str, **kwargs: Dict[str, Any]) -> Response:
    """Perform search against Spycloud's Investigations API to query its vast collection of
        breach records and surrounding metadata

    Args:
        search_type (str): can be one of domain, email, ip, infected-machine-id, log-id,
            password, username, email-username, phone-number, social-handle, bank-number,
            cc-number, drivers-license, national-id, passport-number, or ssn
        query (str): the search query

    Returns:
        Response: requests.Response object from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'SPYCLOUD_API_INV_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    # These are valid endpoints and their corresponding full URLs. We'll use these
    # to check that the user passed a valid 'search_type' parameter
    base_url: str = 'https://api.spycloud.io/investigations-v2/breach/data'
    valid_endpoints: Dict[str, str] = {
        'domain': f'{base_url}/domains',
        'email': f'{base_url}/emails',
        'ip': f'{base_url}/ips',
        'infected-machine-id': f'{base_url}/infected-machine-ids',
        'log-id': f'{base_url}/log-ids',
        'password': f'{base_url}/passwords',
        'username': f'{base_url}/usernames',
        'email-username': f'{base_url}/email-usernames',
        'phone-number': f'{base_url}/phone-numbers',
        'social-handle': f'{base_url}/social-handles',
        'bank-number': f'{base_url}/bank-numbers',
        'cc-number': f'{base_url}/cc-numbers',
        'drivers-license': f'{base_url}/drivers-licenses',
        'national-id': f'{base_url}/national-ids',
        'passport-number': f'{base_url}/passport-numbers',
        'ssn': f'{base_url}/social-security-numbers',
    }

    # Completely exit if they supply an invalid search_type
    if search_type not in valid_endpoints:
        print(f'[!] Error: "{search_type}" is not a valid search type. Must be one of '
              f'{", ".join(valid_endpoints.keys())}', file=sys.stderr)
        sys.exit(1)

    method: str = 'get'
    url: str = f'{valid_endpoints[search_type]}/{query}'

    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_INV_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        params=params,
        timeout=kwargs.get("timeout")
    )

    return result


from typing import Dict, Any, List
from requests import Response
from urllib.parse import quote
from .broker import make_request
from .helpers import check_required_env_vars, combine_env_configs

env_config: Dict[str, Any] = combine_env_configs()


def ipqs_malicious_url(query: str, **kwargs: Dict[str, Any]) -> Response:
    """IPQualityScore's Malicious URL Scanner API scans links in real-time
        to detect suspicious URLs. Accurately identify phishing links, malware
        URLs and viruses, parked domains, and suspicious URLs with real-time risk
        scores. Industry leading phishing detection and domain reputation provide
        better signals for more accurate decision making.

    Args:
        query (str): The URL to scan

    Returns:
        Response: requests.Response json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'IPQS_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'post'
    url: str = 'https://ipqualityscore.com/api/json/url'
    headers: Dict = {'accept': 'application/json'}
    encoded_query: str = quote(query)
    params: Dict = {
        'key': env_config['IPQS_API_KEY'],
        'url': encoded_query,
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        json=params,
        timeout=kwargs.get("timeout")
    )

    return result



from typing import Dict, Any, List
from requests import Response
from .broker import make_request
from .helpers import check_required_env_vars, combine_env_configs

env_config: Dict[str, Any] = combine_env_configs()


def urlscan_search(query: str, **kwargs: Dict[str, Any]) -> Response:
    """Find archived scans of URLs on urlscan.io. Search query syntax can
        be found at https://urlscan.io/docs/search/

    Args:
        query (str): The query term (ElasticSearch Query String Query). Default: "*"

    Returns:
        Response: requests.Response json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'URLSCAN_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = 'https://urlscan.io/api/v1/search/'
    headers: Dict = {
        'accept': 'application/json',
        'API-Key': env_config['URLSCAN_API_KEY']
    }
    params: Dict = {
        'q': query,
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        params=params,
        timeout=kwargs.get("timeout")
    )

    return result


def urlscan_scan(query: str, **kwargs: Dict[str, Any]) -> Response:
    """Submit a URL to be scanned

    Args:
        query (str): the URL to be scanned

    Returns:
        Response: requests.Response json response from the request
    """

    required_vars: List[str] = [
        'URLSCAN_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'post'
    url: str = 'https://urlscan.io/api/v1/scan'
    headers: Dict = {
        'accept': 'application/json',
        'API-Key': env_config['URLSCAN_API_KEY']
    }
    payload: Dict = {
        'url': query,
        **kwargs
    }

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        json=payload,
        timeout=kwargs.get("timeout")
    )

    return result


def urlscan_results(uuid: str, **kwargs: Dict[str, Any]) -> Response:
    """Retrieve results of a URLScan scan

    Args:
        uuid (str): the UUID of the submitted URL scan

    Returns:
        Response: requests.Response json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'URLSCAN_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://urlscan.io/api/v1/result/{uuid}'
    headers: Dict = {
        'accept': 'application/json',
        'API-Key': env_config['URLSCAN_API_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        params=params,
        timeout=kwargs.get("timeout")
    )

    return result


def urlscan_get_dom(uuid: str, **kwargs: Dict[str, Any]) -> Response:
    """Retrieve the DOM from a scan result

    Args:
        uuid (str): the unique ID of the scan

    Returns:
        Response: requests.Response json response from the request
    """

    method: str = 'get'
    url: str = f'https://urlscan.io/dom/{uuid}'
    headers: Dict = {
        'accept': 'application/json',
        'API-Key': env_config['URLSCAN_API_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        params=params,
        timeout=kwargs.get("timeout")
    )

    return result


def urlscan_structure_search(uuid: str, **kwargs: Dict[str, Any]) -> Response:
    # Define required environment variables
    required_vars: List[str] = [
        'URLSCAN_API_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://urlscan.io/api/v1/pro/result/{uuid}/similar'
    headers: Dict = {
        'accept': 'application/json',
        'API-Key': env_config['URLSCAN_API_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(
        method=method,
        url=url,
        headers=headers,
        params=params,
        timeout=kwargs.get("timeout")
    )

    return result
